/* PIKA - Photo and Image Kooker Application
 * a rebranding of The GNU Image Manipulation Program (created with heckimp)
 * A derived work which may be trivial. However, any changes may be (C)2023 by Aldercone Studio
 *
 * Original copyright, applying to most contents (license remains unchanged): 
 * Copyright (C) 1995 Spencer Kimball and Peter Mattis
 *
 * pikacontainertreestore.h
 * Copyright (C) 2010 Michael Natterer <mitch@gimp.org>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef __PIKA_CONTAINER_TREE_STORE_H__
#define __PIKA_CONTAINER_TREE_STORE_H__


enum
{
  PIKA_CONTAINER_TREE_STORE_COLUMN_RENDERER,
  PIKA_CONTAINER_TREE_STORE_COLUMN_NAME,
  PIKA_CONTAINER_TREE_STORE_COLUMN_NAME_ATTRIBUTES,
  PIKA_CONTAINER_TREE_STORE_COLUMN_NAME_SENSITIVE,
  PIKA_CONTAINER_TREE_STORE_COLUMN_USER_DATA,
  PIKA_CONTAINER_TREE_STORE_N_COLUMNS
};


#define PIKA_TYPE_CONTAINER_TREE_STORE            (pika_container_tree_store_get_type ())
#define PIKA_CONTAINER_TREE_STORE(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), PIKA_TYPE_CONTAINER_TREE_STORE, PikaContainerTreeStore))
#define PIKA_CONTAINER_TREE_STORE_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), PIKA_TYPE_CONTAINER_TREE_STORE, PikaContainerTreeStoreClass))
#define PIKA_IS_CONTAINER_TREE_STORE(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), PIKA_TYPE_CONTAINER_TREE_STORE))
#define PIKA_IS_CONTAINER_TREE_STORE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PIKA_TYPE_CONTAINER_TREE_STORE))
#define PIKA_CONTAINER_TREE_STORE_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), PIKA_TYPE_CONTAINER_TREE_STORE, PikaContainerTreeStoreClass))


typedef struct _PikaContainerTreeStoreClass PikaContainerTreeStoreClass;

struct _PikaContainerTreeStore
{
  GtkTreeStore  parent_instance;
};

struct _PikaContainerTreeStoreClass
{
  GtkTreeStoreClass  parent_class;
};


GType          pika_container_tree_store_get_type      (void) G_GNUC_CONST;

void           pika_container_tree_store_columns_init  (GType                  *types,
                                                        gint                   *n_types);
gint           pika_container_tree_store_columns_add   (GType                  *types,
                                                        gint                   *n_types,
                                                        GType                   type);

GtkTreeModel * pika_container_tree_store_new           (PikaContainerView      *container_view,
                                                        gint                    n_columns,
                                                        GType                  *types);

void       pika_container_tree_store_add_renderer_cell (PikaContainerTreeStore *store,
                                                        GtkCellRenderer        *cell,
                                                        gint                    column_number);
PikaViewRenderer *
               pika_container_tree_store_get_renderer  (PikaContainerTreeStore *store,
                                                        GtkTreeIter            *iter);

void           pika_container_tree_store_set_use_name  (PikaContainerTreeStore *store,
                                                        gboolean                use_name);
gboolean       pika_container_tree_store_get_use_name  (PikaContainerTreeStore *store);

void           pika_container_tree_store_set_context   (PikaContainerTreeStore *store,
                                                        PikaContext            *context);
GtkTreeIter *  pika_container_tree_store_insert_item   (PikaContainerTreeStore *store,
                                                        PikaViewable           *viewable,
                                                        GtkTreeIter            *parent,
                                                        gint                    index);
void           pika_container_tree_store_remove_item   (PikaContainerTreeStore *store,
                                                        PikaViewable           *viewable,
                                                        GtkTreeIter            *iter);
void           pika_container_tree_store_reorder_item  (PikaContainerTreeStore *store,
                                                        PikaViewable           *viewable,
                                                        gint                    new_index,
                                                        GtkTreeIter            *iter);
gboolean       pika_container_tree_store_rename_item   (PikaContainerTreeStore *store,
                                                        PikaViewable           *viewable,
                                                        GtkTreeIter            *iter);
void           pika_container_tree_store_clear_items   (PikaContainerTreeStore *store);
void           pika_container_tree_store_set_view_size (PikaContainerTreeStore *store);


#endif  /*  __PIKA_CONTAINER_TREE_STORE_H__  */
